unit fOCSession;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, fOCMonograph,
  fAutoSz, StdCtrls, ORNet, ORFn, uConst, ExtCtrls, ORCtrls, VA508AccessibilityManager,
  Grids, strUtils, uDlgComponents, VAUtils, VA508AccessibilityRouter,
  Vcl.ComCtrls, Winapi.RichEdit, ShellAPI;

type

  TTextRange = record
     chrg: TCharRange;
     lpstrText: PWideChar;
    end;

  PENLink = ^TENLink;

  // Extension of the TRichEdit
  TRichEdit = class(Vcl.ComCtrls.TRichEdit)
  private
    FAutoDetect: Boolean; // URL Auto Detection
    FClickRange: TCharRange; // Range when left mouse button clicked
    FRchState: TObject; // Holds text for recreate
    procedure CNNotify(var Msg: TWMNotify); message CN_NOTIFY;
    function GetAutoURLDetect: Boolean;
    function GetTextRange(StartPos, EndPos: Longint): string;
    procedure SetAutodetect(aValue: Boolean);
    procedure WMSetText(var Msg: TMessage); message WM_SETTEXT;
  protected
    procedure CreateWnd; override;
    procedure DestroyWnd; override;
  public
    property AutoDetect: Boolean read GetAutoURLDetect write SetAutodetect
      default true;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy(); override;
  end;

  TRichEditState = class(TObject)
  private
    FModified: Boolean; //Holds the previous modified property
    FSelLength: Integer; //Holds the previous selection length
    FSelStart: Integer; //Holds the previous selection start point
    FStream: TMemoryStream; //Holds the previous text
  public
    constructor Create;
    destructor Destroy; override;
    procedure Restore(RE: TRichEdit);
    procedure Store(RE: TRichEdit);
  end;

  TfrmOCSession = class(TfrmAutoSz)
    lblInstr: TVA508StaticText;
    OC_PanelGroup: TCategoryPanelGroup;
    CheckBox1: TCheckBox;
    memNote: TMemo;
    cmdCancelOrder: TButton;
    cmdContinue: TButton;
    btnReturn: TButton;
    lblOverrideChecks: TLabel;
    cmdMonograph: TButton;
    btnAllergyAssessment: TButton;
    procedure cmdCancelOrderClick(Sender: TObject);
    procedure cmdContinueClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnReturnClick(Sender: TObject);
    procedure cmdMonographClick(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure btnAllergyAssessmentClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    FCancelSignProcess : Boolean;
    FCheckList: TStringList;
    FOrderList: TStringList;
    procedure SetReturn(const Value: Boolean);
    procedure SetCancellationOnAllPanels(flag : Boolean);
  public
    { Public declarations }
    property CancelSignProcess : Boolean read FCancelSignProcess write SetReturn default false;
    procedure CheckAllValidReasons(Sender: TObject);
  end;

procedure ExecuteReleaseOrderChecks(SelectList: TList);
function ExecuteSessionOrderChecks(OrderList: TStringList) : Boolean;

implementation

{$R *.DFM}

uses rOrders, fCover, uCore, rMisc, fFrame;

type
  TOCRec = class
    OrderID: string;
    OrderText: string;
    Checks: TStringList;
    constructor Create(const AnID: string);
    destructor Destroy; override;
  end;

var
  uCheckedOrders: TList;
  cdl, lastcdl: string;
  ORChecks : Integer;
  CleanedList: TList; //used with function cmdContinueClick
  TagList: TStringList; // KCH for NSR#20101203
  tabCnt : Integer;

constructor TOCRec.Create(const AnID: string);
begin
  OrderID := AnID;
  Checks := TStringList.Create;
end;

destructor TOCRec.Destroy;
begin
  Checks.Free;
  inherited Destroy;
end;

{$REGION 'TRichEdit'}


{-------------------------------------------------------------------------------
  Procedure:   TRichEdit.CreateWnd
  Author:      VHAISPBELLC
  DateTime:    2015.08.21
  Arguments:   None
  Result:      None
  Description: Override to the create of the handle
-------------------------------------------------------------------------------}
procedure TRichEdit.CreateWnd;
var
  mask: Word;
begin
  inherited CreateWnd;
  try
    //Enable the use of auto detect url
    mask := SendMessage(Handle, EM_GETEVENTMASK, 0, 0);
    SendMessage(Handle, EM_SETEVENTMASK, 0, mask or ENM_LINK);
    SendMessage(Handle, EM_AUTOURLDETECT, Integer(true), 0);
    fAutoDetect := true;

    //If restoring the handle then stream in the previous text
    if Assigned(FRchState) then
    begin
      TRichEditState(FRchState).Restore(Self);
      FreeAndNil(FRchState);
    end;
  except
    fAutoDetect := false;
  end;
end;


{-------------------------------------------------------------------------------
  Procedure:   TRichEdit.DestroyWnd
  Author:      VHAISPBELLC
  DateTime:    2015.08.21
  Arguments:   None
  Result:      None
  Description: Override to the destroy of the handle
-------------------------------------------------------------------------------}
procedure TRichEdit.DestroyWnd;
begin
  if csRecreating in ControlState then
  begin
    //If recreating the handle then store off the text
    FRchState := TRichEditState.Create;
    TRichEditState(FRchState).Store(Self);
  end;

  inherited DestroyWnd;
end;


{-------------------------------------------------------------------------------
  Procedure:   TRichEdit.SetAutodetect
  Author:      VHAISPBELLC
  DateTime:    2015.08.21
  Arguments:   aValue: Boolean
  Result:      None
  Description: Setter for AutoDetect. Allow the auto detect url functionality to be
               toggled on or off
-------------------------------------------------------------------------------}
procedure TRichEdit.SetAutodetect(aValue: Boolean);
begin
  if aValue <> fAutoDetect then
  begin
    fAutoDetect := aValue;
    if HandleAllocated then
      SendMessage(Handle, EM_AUTOURLDETECT, WPARAM(fAutoDetect), 0);
  end;
end;

{-------------------------------------------------------------------------------
  Procedure:   TRichEdit.GetAutoURLDetect
  Author:      VHAISPBELLC
  DateTime:    2015.08.21
  Arguments:   aValue: Boolean
  Result:      None
  Description: Getter for AutoDetect
-------------------------------------------------------------------------------}
function TRichEdit.GetAutoURLDetect: Boolean;
begin
  Result := fAutoDetect;
  if HandleAllocated and not(csDesigning in ComponentState) then
  begin
      Result := Boolean(SendMessage(Handle, EM_GETAUTOURLDETECT, 0, 0));
  end;
end;


{-------------------------------------------------------------------------------
  Procedure:   TRichEdit.GetTextRange
  Author:      VHAISPBELLC
  DateTime:    2015.08.21
  Arguments:   StartPos, EndPos: Longint
  Result:      string
  Description: Gets the text between two positions
-------------------------------------------------------------------------------}
function TRichEdit.GetTextRange(StartPos, EndPos: Longint): string;
var
  TextRange: TTextRange;
begin
  SetLength(Result, EndPos - StartPos + 1);
  TextRange.chrg.cpMin := StartPos;
  TextRange.chrg.cpMax := EndPos;
  TextRange.lpstrText := PChar(Result);
  SetLength(Result, SendMessage(Handle, EM_GETTEXTRANGE, 0,
    LPARAM(@TextRange)));
end;

{ -------------------------------------------------------------------------------
  Procedure:   TRichEdit.CNNotify
  Author:      VHAISPBELLC
  DateTime:    2015.08.21
  Arguments:   var Msg: TWMNotify
  Result:      None
  Description: Catch the notify massage
  ------------------------------------------------------------------------------- }
procedure TRichEdit.CNNotify(var Msg: TWMNotify);
begin
  with Msg do
  begin
    case NMHdr^.code of
      EN_LINK:
        with PENLink(NMHdr)^ do
        begin
          case Msg of
            WM_LBUTTONDOWN:
              // capture the click range
              FClickRange := chrg;
            WM_LBUTTONUP:
              begin
                // If we are still within the url region then shell execute the url
                if (FClickRange.cpMin = chrg.cpMin) and
                  (FClickRange.cpMax = chrg.cpMax) then
                  // This could be handled off in it's own event in the future
                  ShellExecute(Handle, 'open',
                    PChar(GetTextRange(chrg.cpMin, chrg.cpMax)), nil, nil,
                    SW_SHOWNORMAL);
                //clear the click range
                with FClickRange do
                begin
                  cpMin := -1;
                  cpMax := -1;
                end;
              end;
          end;
        end;
    end;
    inherited;
  end;
end;

{-------------------------------------------------------------------------------
  Procedure:   TRichEdit.WMSetText
  Author:      VHAISPBELLC
  DateTime:    2015.08.21
  Arguments:   var Msg: TMessage
  Result:      None
  Description: Ensures the handle exist if auto detect url is true
-------------------------------------------------------------------------------}
procedure TRichEdit.WMSetText(var Msg: TMessage);
begin
  if AutoDetect then
    HandleNeeded;
  inherited;
end;

{-------------------------------------------------------------------------------
  Procedure:   TRichEdit.Create
  Author:      VHAISPBELLC
  DateTime:    2015.08.21
  Arguments:   AOwner: TComponent
  Result:      None
  Description: Override to the constructor
-------------------------------------------------------------------------------}
constructor TRichEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FRchState := nil;
end;

{-------------------------------------------------------------------------------
  Procedure:   TRichEdit.Destroy
  Author:      VHAISPBELLC
  DateTime:    2015.08.21
  Arguments:   AOwner: TComponent
  Result:      None
  Description: Override to the destructor
-------------------------------------------------------------------------------}
destructor TRichEdit.Destroy();
begin
  FRchState.Free;
  inherited;
end;

{$ENDREGION}

{$REGION 'TRichEditState'}

{-------------------------------------------------------------------------------
  Procedure:   TRichEditState.Create
  Author:      VHAISPBELLC
  DateTime:    2015.08.21
  Arguments:   None
  Result:      None
  Description: Constructor
-------------------------------------------------------------------------------}
constructor TRichEditState.Create;
begin
  inherited Create;
  FStream := TMemoryStream.Create;
end;

{-------------------------------------------------------------------------------
  Procedure:   TRichEditState.Destroy
  Author:      VHAISPBELLC
  DateTime:    2015.08.21
  Arguments:   None
  Result:      None
  Description: Destructor
-------------------------------------------------------------------------------}
destructor TRichEditState.Destroy;
begin
  FStream.Free;
  inherited Destroy;
end;

{-------------------------------------------------------------------------------
  Procedure:   TRichEditState.Restore
  Author:      VHAISPBELLC
  DateTime:    2015.08.21
  Arguments:   RE: TRichEdit
  Result:      None
  Description: Restores the previous properties/text
-------------------------------------------------------------------------------}
procedure TRichEditState.Restore(RE: TRichEdit);
begin
  FStream.Position := 0; // start of the stream
  RE.Lines.LoadFromStream(FStream); //Load from the stream
  RE.SelStart := FSelStart; //Restore the selection start
  RE.SelLength := FSelLength; //Restore the selection length
  RE.Modified := FModified;  //Restore the modified
end;

{-------------------------------------------------------------------------------
  Procedure:   TRichEditState.Store
  Author:      VHAISPBELLC
  DateTime:    2015.08.21
  Arguments:   RE: TRichEdit
  Result:      None
  Description: Store the previos properties/text
-------------------------------------------------------------------------------}
procedure TRichEditState.Store(RE: TRichEdit);
begin
  FModified := RE.Modified; //Store the previous modified
  FSelStart := RE.SelStart; //Store the previous selection start
  FSelLength := RE.SelLength; //Store the previous selection length
  RE.Lines.SaveToStream(FStream); //Store the previous text
end;

{$ENDREGION}
procedure ExecuteReleaseOrderChecks(SelectList: TList);
var
  i: Integer;
  AnOrder: TOrder;
  OrderIDList: TStringList;

begin
  OrderIDList := TStringList.Create;
  TagList := TStringList.Create;
  CleanedList := TList.Create;
  CleanedList := SelectList;

  try
    for i := 0 to SelectList.Count - 1 do
    begin
      AnOrder := TOrder(SelectList.Items[i]);
      OrderIDList.Add(AnOrder.ID + '^^1');  // 3rd pce = 1 means releasing order
      TagList.AddObject(AnOrder.ID + '^^1', AnOrder);  // 3rd pce = 1 means releasing order
    end;
    while OrderIDList.Count > 0 do
	  begin
      if ExecuteSessionOrderChecks(OrderIDList) then
      begin
        for i := SelectList.Count - 1 downto 0 do
        begin
          AnOrder := TOrder(SelectList.Items[i]);
          if OrderIDList.IndexOf(AnOrder.ID + '^^1') < 0 then
          begin
            Changes.Remove(CH_ORD, AnOrder.ID);
            SelectList.Delete(i);
          end;
        end;
        Break;
      end;
	  end;
    if OrderIDList.Count < 1 then SelectList.Clear;
  finally
    OrderIDList.Free;
  end;
end;

{Returns True if the Signature process should proceed. Clears OrderList If False. }
function ExecuteSessionOrderChecks(OrderList: TStringList) : Boolean;
var
  i, j, n : Integer;
  LastID, NewID, gridText, substr, format: string;
  CheckList: TStringList;
  OCRec: TOCRec;
  frmOCSession: TfrmOCSession;
  newPanel: TCategoryPanel;
  chorder : TRichEdit;

  chkBox : TCheckbox;
  lblJustify, lblID : TLabel;
  txtJustify : TComboBox;
  TCString : String;
  remOC: TStringList;

begin
  chorder := nil;
  ORChecks := 0;
  Result := True;
  CheckList := TStringList.Create;
  remOC := TStringList.Create;

  try
    StatusText('Order Checking...');
    OrderChecksForSession(CheckList, OrderList);
    StatusText('');
    tabCnt := 0;

    if CheckList.Count > 0 then
    begin
      frmOCSession := TfrmOCSession.Create(Application);
    // *** The following 3 lines of code are for future use when an // NJC(NSR 20070920) 08/15
    // *** authorization parameter is implemented for Allergy access// NJC(NSR 20070920) 08/15
    //  if sCallV('ORQQAE USER',[user.DUZ]) <> '1' then             // NJC(NSR 20070920) 08/15
    //    frmOCAccept.AllergyAssessmentBtn.Visible  := false        // NJC(NSR 20070920) 08/15
    //  else                                                        // NJC(NSR 20070920) 08/15
      if sCallV('ORQQAL LIST', [Patient.DFN]) = '^No Allergy Assessment' then //NJC 09/15
        frmOCSession.btnAllergyAssessment.Visible := true           // NJC(NSR 20070920) 08/15
       else                                                         // NJC(NSR 20070920) 08/15
        frmOCSession.btnAllergyAssessment.Visible := false;         // NJC(NSR 20070920) 08/15
      frmOCSession.cmdMonograph.Enabled := false;
      if IsMonograph then frmOCSession.cmdMonograph.Enabled := true;
      try
        ResizeFormToFont(TForm(frmOCSession));
        uCheckedOrders := TList.Create;
        LastID := '';

        for i := 0 to CheckList.Count - 1 do
        begin
//*****************************************************************************
//   Added Section on 12/22/2014 to 12/26/2014 by KCH for NSR# 20101203
//*****************************************************************************
          NewID := Piece(CheckList[i], U, 1);
          format := Piece(CheckList[i], U, 2);
          cdl := trim(Piece(CheckList[i], U, 3));
          gridText := Piece(CheckList[i], U, 4);

//          if (NewID <> LastID) OR ((NewID = LastID) AND (cdl <> lastcdl)) then
          if NewID <> LastID then
          begin
            OCRec := TOCRec.Create(NewID);
            uCheckedOrders.Add(OCRec);
            LastID := NewID;
            lastcdl := cdl;
            Inc(tabCnt);
            TCString := '('+IntToStr(tabCnt)+')  '+trim(StringReplace(TextForOrder(NewID),#$D#$A,'',[rfReplaceAll, rfIgnoreCase]));
            TCString := trim(StringReplace(TCString,'*UNSIGNED*','',[rfReplaceAll, rfIgnoreCase]));

            newPanel := frmOCSession.OC_PanelGroup.CreatePanel(frmOCSession.OC_PanelGroup) as TCategoryPanel;
            newPanel.Name := 'ocpg_'+IntToStr(i+1);
            newPanel.Tag := i;
            newPanel.Caption := TCString + ' ' + IntToStr(i+1);
            newPanel.Height := 165;
            newPanel.Expand;

            chkBox := TCheckBox.Create(newPanel);
            chkBox.parent := newPanel;
            chkBox.TabStop := False;
            chkBox.Tag := i;
            chkBox.left := 3;
            chkBox.Top := 3;
            chkBox.Width := 200;
            chkBox.Name := 'canOrder_'+IntToStr(i+1);
            chkBox.Caption := 'Cancel Order? ';

            lblID := TLabel.Create(newPanel);
            lblID.parent := newPanel;
            lblID.Width := 75;
            lblID.Height := 25;
            lblID.left := 100;
            lblID.Top := 5;
            lblID.Name := 'lblID_'+IntToStr(i+1);
            lblID.Visible := False;
            lblID.Caption := NewID;

            chorder := TRichEdit.Create(nil);
            chorder.parent := newPanel;
            chorder.ScrollBars := ssVertical;
            chorder.TabStop := False;
            chorder.Width := newPanel.Width-25; //or use 790
            chorder.left := 3;
            chorder.Top := 23;
            chorder.Name := 'rtfOrder_'+IntToStr(i+1);
            chorder.Clear;
            chorder.AutoDetect := True;

            if cdl = '1' then
            begin
              Inc(ORChecks);
              txtJustify := TComboBox.Create(newPanel);
              txtJustify.parent := newPanel;
              txtJustify.Style := csDropDown;
              txtJustify.Height := 80;
              txtJustify.Name := 'txtJustify_'+IntToStr(i+1);
              txtJustify.Text := '';
              txtJustify.TabStop := True;
              txtJustify.TabOrder := ORChecks;
              txtJustify.Tag := Integer(TagList.Objects[ORChecks-1]);

              lblJustify := TLabel.Create(newPanel);
              lblJustify.parent := newPanel;
              lblJustify.AutoSize := True;
              lblJustify.Height := 25;
              lblJustify.left := 3;
              lblJustify.Top := chorder.Top + chorder.Height + 4;
              lblJustify.Name := 'lblJustify_'+IntToStr(i+1);
              lblJustify.Caption := 'Reason for overriding order checks:';

              txtJustify.Top := lblJustify.Top - 2;
              txtJustify.Left := lblJustify.Left + lblJustify.Width + 10;
              txtJustify.Width := chorder.Width - (txtJustify.Left-4); //610;

              GetAllergyReasonList(StrToInt(Piece(NewID,';', 1)),'O');
              FastAssign(RPCBrokerV.Results,txtJustify.Items);
              txtJustify.onChange := frmOCSession.CheckAllValidReasons;
            end;
          end; {if NewID or new cdl}

//          chorder.Lines.Add(hypertext);

          if cdl = '1' then
          begin
            chorder.SelAttributes.Color := Get508CompliantColor(clBlue);
          end
          else if cdl = '2' then
          begin
            chorder.SelAttributes.Color := Get508CompliantColor(clGreen);
          end
          else
          begin
            chorder.SelAttributes.Color := Get508CompliantColor(clBlack);
          end;

          substr := Copy(Piece(CheckList[i], U, 4), 0, 2);
          if substr = '||' then
            begin
              gridText := '';
              substr := Copy(Piece(CheckList[i], U, 4), 3, Length(Piece(CheckList[i], U, 4)));
              GetXtraTxt(remOC, Piece(substr, '&', 1), Piece(substr, '&', 2));
              chorder.Lines.Add('(' + inttostr(i + 1) + ' of ' + inttostr(CheckList.Count) + ')  ');
              for j := 0 to remOC.Count - 1 do
                chorder.Lines.Add('      ' + remOC[j]);
            end
          else
            begin
              chorder.Lines.Add(gridText + CRLF);
            end;
        end;
//******************************************************************************
//END NEW SECTION
//******************************************************************************
        try
          frmOCSession.lblOverrideChecks.Caption :=
            '('+IntToStr(ORChecks)+') Total Critical High Order Overrides not Entered';
        finally
        end;

        frmOCSession.FOrderList := OrderList;
        frmOCSession.FCheckList := CheckList;
        MessageBeep(MB_ICONASTERISK);
        if frmOCSession.Visible then frmOCSession.SetFocus;
        frmOCSession.ShowModal;
        Result := not frmOCSession.CancelSignProcess;
        if frmOCSession.CancelSignProcess then begin
          for n := 0 to OrderList.Count - 1 do
            UnlockOrder(Piece(OrderList[n], U, 1));
          OrderList.Clear;
          if Assigned(frmFrame) then
            frmFrame.SetActiveTab(CT_ORDERS);
        end
		    else
		    if frmOCSession.modalresult = mrRetry then Result := False;
      finally
        with uCheckedOrders do for i := 0 to Count - 1 do TOCRec(Items[i]).Free;
        frmOCSession.Free;
        remOC.Destroy;
      end; {try}
    end; {if CheckList}
  finally
    CheckList.Free;
    TagList.Free;
  end;
end;

//Re-Written by KCH for NSR#20101203 01/05/2015 and 01/06/2015
//Modified by KCH for NSR#20101203 07/17/2015 for major interfeace
//design changes in the form
procedure TfrmOCSession.cmdCancelOrderClick(Sender: TObject);
var
  i,x,j,cntfound : Integer;
  isfound: Boolean;
  AnOrderID: string;
  TmpPanel : TCategoryPanel;
  surface: TCategoryPanelSurface;

begin
  cntfound := 0;
  for i := 0 to OC_PanelGroup.ControlCount-1 do
  begin
    isfound := False;
    AnOrderID := '';
    if OC_PanelGroup.Controls[i] is TCategoryPanel then //Loop through the Panel
      begin
        TmpPanel := TCategoryPanel(OC_PanelGroup.Controls[i]);
        surface := TmpPanel.Controls[0] as TCategoryPanelSurface;

        for j := 0 to surface.ControlCount - 1 do
        begin
          if (surface.Controls[j].ClassName = 'TCheckBox') then
          begin
      //      Check the Checkbox for check status of True
            if (surface.Controls[j] as TCheckBox).Checked then isfound := True;
          end;

          if surface.Controls[j].ClassName = 'TLabel' then
            if AnsiPos('lblID_', (surface.Controls[j] as TLabel).Name) > 0 then
            begin
              // Get the OrderID from a hidden label on the Panel surface component
              AnOrderID := (surface.Controls[j] as TLabel).Caption;
            end;

          if (AnOrderID <> '') AND (isfound) then
          begin
            Inc(cntfound);
            for x := FCheckList.Count - 1 downto 0 do
              if Piece(FCheckList[x], U, 1) = AnOrderID then
                FCheckList.Delete(x);
          end;
        end;
      end;

      if (cntfound > 0) AND (AnOrderID <> '') AND (DeleteCheckedOrder(AnOrderID)) then
      begin
        Changes.Remove(CH_ORD, AnOrderId);
      end;
  end;

  if cntfound = 0 then
  begin
    if (ScreenReaderActive) then GetScreenReader.Speak('No orders are marked to cancel. Check the Cancel box by the orders to cancel.');
	  infoBox('No orders are marked to cancel. Check the Cancel box by the orders to cancel.', 'Error', MB_OK);
  end;
end;

procedure TfrmOCSession.cmdContinueClick(Sender: TObject);
var
i, j, SCnt, Ind: integer;
Cancel: boolean;
TmpPanel : TCategoryPanel;
surface: TCategoryPanelSurface;
AnOrderID, ReasonStr : String;
OList: TStringList;

begin
  inherited;
  Cancel := False;
  OList := TStringList.Create;

//******************************************************************************
// Added by KCH 01/08/2015 and Modified 07/21/2015 for NSR#20101203
//******************************************************************************
  for i := 0 to OC_PanelGroup.ControlCount-1 do
  begin
    if OC_PanelGroup.Controls[i] is TCategoryPanel then //Loop through the Panel
      begin
        TmpPanel := TCategoryPanel(OC_PanelGroup.Controls[i]);
        surface := TmpPanel.Controls[0] as TCategoryPanelSurface;

        for j := 0 to surface.ControlCount - 1 do
        begin
          if (surface.Controls[j].ClassName = 'TCheckBox') then
          begin
      //      Check the Checkbox for check status of True
            if (surface.Controls[j] as TCheckBox).Checked then
            begin
              Cancel := True;
              Break;
            end;
          end;

          if surface.Controls[j].ClassName = 'TComboBox' then
          begin
            if (((Trim((surface.Controls[j] as TComboBox).Text)) <> '') AND
                (Length(Trim((surface.Controls[j] as TComboBox).Text)) < 4)) then
            begin
               InfoBox('A valid justification for overriding critical order checks is required.',
                      'Justification Required', MB_OK);
              Exit;
            end;

            if ContainsUpCarretChar((surface.Controls[j] as TComboBox).Text) then
            begin
               InfoBox('The justification may not contain the ^ character.',
                      'Justification Required', MB_OK);
              Exit;
            end;
          end;
        end;

        if Cancel then Break;
      end;
  end;

  if Cancel = True then
    begin
      InfoBox('One or more orders have been marked to cancel!' + CRLF + CRLF +
        'To cancel these orders, click the "Cancel Checked Order(s)" button.' + CRLF + CRLF +
        'To place these orders, uncheck the Cancel box beside the order you wish to keep and then click the "Accept Order(s)" button again.',
        'Error', MB_OK);
      OList.Destroy;
      Exit;
    end;

  for i := 0 to OC_PanelGroup.ControlCount - 1 do
  begin
    if OC_PanelGroup.Controls[i] is TCategoryPanel then //Loop through the TCategoryPanel
    begin
        ReasonStr := '';
        AnOrderID := '';
        TmpPanel := TCategoryPanel(OC_PanelGroup.Controls[i]);
        surface := TmpPanel.Controls[0] as TCategoryPanelSurface;
        SCnt := surface.ControlCount;

        for j := 0 to SCnt - 1 do
        begin
          if surface.Controls[j].ClassName = 'TComboBox' then
          begin
            ReasonStr := Trim((surface.Controls[j] as TComboBox).Text);
          end;

          if (surface.Controls[j].ClassName = 'TLabel') AND
             (AnsiPos('lblID_', (surface.Controls[j]).Name) > 0) then
          begin
            AnOrderID := (surface.Controls[j] as TLabel).Caption;
          end;

          if (surface.Controls[j].ClassName = 'TComboBox') AND
            (ReasonStr <> '') AND (AnOrderID <> '') AND (j = 4) then
          begin
            StatusText('Saving Order Check...');
            OList.Add(AnOrderID);
            SaveOrderChecksForSession(ReasonStr, OList);
            StatusText('');
            OList.Clear;
          end
          else
          if (surface.Controls[j].ClassName = 'TComboBox') AND
            (ReasonStr = '') AND (AnOrderID <> '') AND (j = 4) then
          begin
            //Remove Item from Selected ListA
            Ind := (surface.Controls[j] as TComboBox).Tag;
            Ind := CleanedList.IndexOf(Pointer(ind));
            CleanedList.Delete(Ind);
          end
          else
          if (j = 2) AND (SCnt = 3) AND (ReasonStr = '') then
          begin
            StatusText('Saving Order Check...');
            OList.Add(AnOrderID);
            SaveOrderChecksForSession('Not Critical', OList);
            StatusText('');
            OList.Clear;
          end;
        end;
    end;
  end;

  OList.Destroy;
//******************************************************************************
// END NEW BLOCK
//******************************************************************************
  Close;
end;

procedure TfrmOCSession.cmdMonographClick(Sender: TObject);
var
  monoList: TStringList;
begin
  inherited;
  monoList := TStringList.Create;
  GetMonographList(monoList);
  ShowMonographs(monoList);
  monoList.Free;
end;


procedure TfrmOCSession.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  inherited;
  SaveUserBounds(Self); //Save Position & Size of Form
  DeleteMonograph;
end;

procedure TfrmOCSession.FormShow(Sender: TObject);
begin
  inherited;
  memNote.TabOrder := ORChecks+1;
  cmdCancelOrder.TabOrder := memNote.TabOrder+1;
  cmdContinue.TabOrder := cmdCancelOrder.TabOrder+1;
  btnReturn.TabOrder := cmdContinue.TabOrder+1;
  cmdMonograph.TabOrder := btnReturn.TabOrder+1;
  btnAllergyAssessment.TabOrder := cmdMonograph.TabOrder+1;
  if tabCnt < 3 then
    ClientHeight := (tabCnt * 165) + 141
  else
    ClientHeight := (3 * 165) + 141;
end;

//************ ADDED BY KCH 08/14/2015 ********************
// Had to separate the Populating of the RichEdit URL text from the
// rest of the code to issues with hyperlinking failing with early population
// and this was accomplished by populating in the form onShow Event
//****************************************************************
procedure TfrmOCSession.SetReturn(const Value: Boolean);
begin
  FCancelSignProcess := Value;
end;

//******************************************************************************
// THIS SECTION BELOW Added by KCH for NSR# 20101203 12/29/2014 to 12/30/2014
//******************************************************************************
procedure TfrmOCSession.btnReturnClick(Sender: TObject);
begin
  inherited;
  FCancelSignProcess := True;
  Close;
end;

//Added by KCH 12/30/2014
procedure TfrmOCSession.SetCancellationOnAllPanels(flag : Boolean);
var
 i,j : Integer;
 TmpPanel : TCategoryPanel;
 surface: TCategoryPanelSurface;

begin
  for i := 0 to OC_PanelGroup.ControlCount - 1 do
  begin
    TmpPanel := TCategoryPanel(OC_PanelGroup.Panels[i]);
    surface := TmpPanel.Controls[0] as TCategoryPanelSurface;
    for j := 0 to surface.ControlCount - 1 do
    begin
      if (surface.Controls[j].ClassName = 'TCheckBox') then
      begin
  //      Set the Checkbox to true so order may be cancelled
        (surface.Controls[j] as TCheckBox).Checked := flag;
        if (ScreenReaderActive) then GetScreenReader.Speak('Cancel checkbox checked');
      end;
    end;
  end;
end;

//Added by KCH 01/08/2015
procedure TfrmOCSession.CheckBox1Click(Sender: TObject);
begin
  inherited;
  if CheckBox1.Checked then
    begin
    CheckBox1.Caption := 'Un-Cancel All Orders?';
    end
  else
    begin
      CheckBox1.Caption := 'Cancel All Orders?';
    end;

  SetCancellationOnAllPanels(CheckBox1.Checked);
end;

// Modified my Validattion to loop through all TComboBox components
// on the GroupPanel component to make sure valid reasons have been
// assigned or not - Added on 08/14/2015
procedure TfrmOCSession.btnAllergyAssessmentClick(Sender: TObject);
begin
  inherited;                                           // NJC(NSR 20070920) 08/15
  frmCover.popNewAllergyClick(Self);                   // NJC(NSR 20070920) 08/15
end;

procedure TfrmOCSession.CheckAllValidReasons(Sender: TObject);
var
  i,j : Integer;
  TmpPanel : TCategoryPanel;
  surface: TCategoryPanelSurface;

begin
  ORChecks := 0;
  for i := 0 to OC_PanelGroup.ControlCount-1 do
  begin
    if OC_PanelGroup.Controls[i] is TCategoryPanel then //Loop through the Panel
      begin
        TmpPanel := TCategoryPanel(OC_PanelGroup.Controls[i]);
        surface := TmpPanel.Controls[0] as TCategoryPanelSurface;

        for j := 0 to surface.ControlCount - 1 do
        begin
          if surface.Controls[j].ClassName = 'TComboBox' then
          begin
            Inc(ORChecks);
            if ((Length((surface.Controls[j] as TComboBox).Text) > 0) AND
                (Length((surface.Controls[j] as TComboBox).Text) < 4)) then
            begin
            end
            else
            if (ContainsUpCarretChar((surface.Controls[j] as TComboBox).Text) = true) then
            begin
            end
            else
              Dec(ORChecks);
          end;
        end;
      end;
  end;
  if ORChecks < 0 then ORChecks := 0;

  lblOverrideChecks.Caption :=
     '('+IntToStr(ORChecks)+') Total Critical High Order Overrides Not Entered';
end;

end.
